/*---------------------------------------------------------------------*/
/* FrGetVect.c          by B. Mours Caltech&LAPP   May  2, 2002        */
/*                                                                     */
/* This Matlab mex file extract from a frame file the data for one     */ 
/* vector (ADC ot Proc or Sim data) (this is for Matlab version 5)     */
/*                                                                     */
/* The input arguments are:                                            */
/*   1) file name                                                      */
/*   2) vector name                                                    */
/*   3) (optional) starting GPS time(default= first frame in the file) */
/*   4) (optional) number of frame (default = 1 Frames)                */
/*                                                                     */
/* Returned matlab data:                                               */
/*   1) ADC data (time serie stored as double)                         */
/*   2) (optional) time values relative to the first data point(double)*/
/*   3) (optional) frequency values (for FFT's) (double)               */
/*   4) (optional) GPS starting time (in second, stored in double)     */
/*   5) (optional) starting time as a string                           */
/*   6) (optional) vector unitX as a string                            */
/*   7) (optional) vector unitY as a string                            */
/*      All this values are stored as double                           */
/*---------------------------------------------------------------------*/
#include "mex.h"
#include "FrameL.h"

void  mexFunction( int nlhs,       mxArray *plhs[],
                   int nrhs, const mxArray *prhs[])
{struct FrFile *iFile;
 struct FrVect *vect;
 int   status;
 long  debugLevel, i, nData;
 double *data, dt, df, t0, length;
 char  *name, fileName[100],vectName[100], stringTime[30];
 time_t tp;

    /*--------------- check arguments ---------------------*/

 if(nrhs<2)
    {mexErrMsgTxt("Please provide the file name and ADC name as argument\n");
     return;}

 if(!mxIsChar(prhs[0]))
    {mexErrMsgTxt("The first argument is not a filename\n");
     return;}
  
 if(!mxIsChar(prhs[1]))
    {mexErrMsgTxt("The second argument is not the vect name \n");
     return;}
  
 if(nlhs==0)
    {mexErrMsgTxt("Please provide at least one output argument\n");
     return;}

 if(nrhs > 2)
     {data = mxGetPr(prhs[2]);
      t0 = data[0];}
 else{t0 = 0;}

 if(nrhs > 3)
     {data = mxGetPr(prhs[3]);
      length = data[0];}
 else{length = 1.;}

    /*----------- open file --------------------------*/

  status=mxGetString(prhs[0],fileName,100);
  status=mxGetString(prhs[1],vectName,100);
  mexPrintf(" Open file:%s for channel:%s t0=%.2f length=%.1f\n",
                   fileName, vectName, t0, length);

  iFile = FrFileINew(fileName);
  if(iFile == NULL)
     {mexPrintf("Error during file opening\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return;}

     /*--------- get the vector ------------------*/
     
  vect = FrFileIGetV(iFile, vectName,  t0, length);
  FrVectDump(vect, stdout, 2);
  if(vect == NULL)
      {mexPrintf(" Vector not found \n");
       FrFileIEnd(iFile); 
       return;}

  nData = vect->nData;
  dt = vect->dx[0];
  df = 1./dt/nData;
  plhs[0] = mxCreateDoubleMatrix(nData,1,mxREAL);
 
    /*------ fill time and frequency array ---------------*/

  if(nlhs > 1)
    {plhs[1] = mxCreateDoubleMatrix(nData,1,mxREAL);
     data = mxGetPr(plhs[1]);
     for(i=0; i<nData; i++)
        {data[i] = vect->startX[0]+(double)i*dt;}}

  if(nlhs > 2)
    {plhs[2] = mxCreateDoubleMatrix(nData/2,1,mxREAL);
     data = mxGetPr(plhs[2]);
     for(i=0; i<nData/2; i++)
        {data[i] = (double)i*df;}}

  if(nlhs > 3)
    {plhs[3] = mxCreateDoubleMatrix(1,1,mxREAL);
     data = mxGetPr(plhs[3]);
     data[0] = (double)vect->GTime;}

  if(nlhs > 4)
    {tp = vect->GTime + 315961200;
     strncpy(stringTime,ctime(&tp),24);
     name = mxCalloc(80,sizeof(char));
     sprintf(name,"Starting GPS time:%s\n",
                            stringTime);
     plhs[4] = mxCreateString(name);
     mxFree(name);}	

  if(nlhs > 5)
    {if(vect->unitX[0] != NULL)
         {name = mxCalloc(strlen(vect->unitX[0])+1,sizeof(char));
          strcpy(name, vect->unitX[0]);
          plhs[5] = mxCreateString(name);
          mxFree(name);}
     else{plhs[5] = mxCreateString("no unit");}}

  if(nlhs > 6)
    {if(vect->unitY != NULL)
         {name = mxCalloc(strlen(vect->unitY)+1,sizeof(char));
          strcpy(name, vect->unitY);
          plhs[6] = mxCreateString(name);
          mxFree(name);}
     else{plhs[6] = mxCreateString("no unit");}}

    /*-------- copy data ------*/

  data = mxGetPr(plhs[0]);
  if(vect->type == FR_VECT_C)
        {for(i=0; i<nData; i++) {data[i] = vect->data[i];}}
  else if(vect->type == FR_VECT_2S)
        {for(i=0; i<nData; i++) {data[i] = vect->dataS[i];}}
  else if(vect->type == FR_VECT_4S)
        {for(i=0; i<nData; i++) {data[i] = vect->dataI[i];}}
  else if(vect->type == FR_VECT_1U)
        {for(i=0; i<nData; i++) {data[i] = vect->dataU[i];}}
  else if(vect->type == FR_VECT_2U)
        {for(i=0; i<nData; i++) {data[i] = vect->dataUS[i];}}
  else if(vect->type == FR_VECT_4U)
        {for(i=0; i<nData; i++) {data[i] = vect->dataUI[i];}}
  else if(vect->type == FR_VECT_4R)
        {for(i=0; i<nData; i++) {data[i] = vect->dataF[i];}}
  else if(vect->type == FR_VECT_8R)
        {for(i=0; i<nData; i++) {data[i] = vect->dataD[i];}}

  FrVectFree(vect);
    
  FrFileIEnd(iFile);            /* Close file and release descriptor */

}
